@@ -1,4 +1,4 @@
-/* Copyright 1999 by Marco d'Itri <md@linux.it>.
+/* Copyright 1999-2003 by Marco d'Itri <md@linux.it>.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -188,7 +188,29 @@ int main(int argc, char *argv[])
 		close(sockfd);
 		if (!server)
 		    exit(0);
-		printf(_("\nFound crsnic referral to %s.\n\n"), server);
+		printf(_("\nFound a referral to %s.\n\n"), server);
+		alarm(60);
+		break;
+	    case 9:
+		if (verb)
+		    puts(_("Connecting to whois.nic.cc."));
+		sockfd = openconn("whois.nic.cc", NULL);
+		server = query_crsnic(sockfd, qstring);
+		close(sockfd);
+		if (!server)
+		    exit(0);
+		printf(_("\nFound a referral to %s.\n\n"), server);
+		alarm(60);
+		break;
+	    case 7:
+		if (verb)
+		    puts(_("Connecting to whois.publicinterestregistry.net."));
+		sockfd = openconn("whois.publicinterestregistry.net", NULL);
+		server = query_pir(sockfd, qstring);
+		close(sockfd);
+		if (!server)
+		    exit(0);
+		printf(_("\nFound referral to %s.\n\n"), server);
 		alarm(60);
 		break;
 	    case 5:
@@ -307,19 +329,19 @@ const char *whichwhois(const char *s)
 	    for (i = 0; ip6_assign[i].serv; i++)
 		if (v6net == ip6_assign[i].net)
 		    return ip6_assign[i].serv;
-	    return "\006";			/* unknown allocation */
+	    return "\x06";			/* unknown allocation */
 	} else if (strncasecmp(s, "3ffe:", 5) == 0)
 	    return "whois.6bone.net";
-	/* RPSL objects like AS8627:fltr-TRANSIT-OUT */
+	/* RPSL hierarchical object like AS8627:fltr-TRANSIT-OUT */
 	else if (strncasecmp(s, "as", 2) == 0 && isasciidigit(s[2]))
 	    return whereas(atoi(s + 2));
 	else
-	    return "\005";
+	    return "\x05";
     }
 
     /* email address */
     if (strchr(s, '@'))
-	return "\005";
+	return "\x05";
 
     /* no dot and no hyphen means it's a NSI NIC handle or ASN (?) */
     if (!strpbrk(s, ".-")) {
@@ -334,7 +356,7 @@ const char *whichwhois(const char *s)
 	if (*s == '!')	/* NSI NIC handle */
 	    return "whois.networksolutions.com";
 	else
-	    return "\005";	/* probably a unknown kind of nic handle */
+	    return "\x05";	/* probably a unknown kind of nic handle */
     }
 
     /* smells like an IP? */
@@ -342,7 +364,7 @@ const char *whichwhois(const char *s)
 	for (i = 0; ip_assign[i].serv; i++)
 	    if ((ip & ip_assign[i].mask) == ip_assign[i].net)
 		return ip_assign[i].serv;
-	return "\005";			/* not in the unicast IPv4 space */
+	return "\x05";			/* not in the unicast IPv4 space */
     }
 
     /* check the TLDs list */
@@ -360,9 +382,9 @@ const char *whichwhois(const char *s)
 	return "";
     }
 
-    /* has dot and maybe a hypen and it's not in tld_serv[], WTF is it? */
+    /* has dot and maybe a hyphen and it's not in tld_serv[], WTF is it? */
     /* either a TLD or a NIC handle we don't know about yet */
-    return "\005";
+    return "\x05";
 }
 
 const char *whereas(const unsigned short asn)
@@ -372,7 +394,7 @@ const char *whereas(const unsigned short asn)
     for (i = 0; as_assign[i].serv; i++)
 	if (asn >= as_assign[i].first && asn <= as_assign[i].last)
 	    return as_assign[i].serv;
-    return "\006";
+    return "\x06";
 }
 
 char *queryformat(const char *server, const char *flags, const char *query)
@@ -406,12 +428,11 @@ char *queryformat(const char *server, const char *flags, const char *query)
 	    puts(_("Warning: RIPE flags used with a traditional server."));
 	strcat(buf, flags);
     }
-    if (!isripe && strcmp(server, "whois.nic.mil") == 0 &&
+    /* FIXME: /e is not applied to .JP ASN */
+    if (!isripe && (strcmp(server, "whois.nic.mil") == 0 ||
+	    strcmp(server, "whois.nic.ad.jp") == 0) &&
 	    strncasecmp(query, "AS", 2) == 0 && isasciidigit(query[2]))
 	sprintf(buf, "AS %s", query + 2);	/* fix query for DDN */
-    if (!isripe && strcmp(server, "whois.arin.net") == 0 &&
-	    strncasecmp(query, "AS", 2) == 0 && isasciidigit(query[2]))
-	sprintf(buf, "A %s", query + 2);	/* always ask for a ASN */
     else if (!isripe && strcmp(server, "whois.corenic.net") == 0)
 	sprintf(buf, "--machine %s", query);	/* machine readable output */
     else if (!isripe && strcmp(server, "whois.nic.ad.jp") == 0) {
@@ -509,11 +530,55 @@ const char *query_crsnic(const int sock, const char *query)
 	    for (p = buf; *p != ':'; p++);	/* skip until colon */
 	    for (p++; *p == ' '; p++);		/* skip colon and spaces */
 	    ret = malloc(strlen(p) + 1);
+	    for (q = ret; *p != '\n' && *p != '\r' && *p != ' '; *q++ = *p++)
+		; /*copy data*/
+	    *q = '\0';
+	    state = 2;
+	}
+	if (verb)
+	    fputs(buf, stdout);
+    }
+    if (ferror(fi))
+	err_sys("fgets");
+
+    free(temp);
+    return ret;
+}
+
+const char *query_pir(const int sock, const char *query)
+{
+    char *temp, buf[2000], *ret = NULL;
+    FILE *fi;
+    int state = 0;
+
+    temp = malloc(strlen(query) + 5 + 2 + 1);
+    strcpy(temp, "FULL ");
+    strcat(temp, query);
+    strcat(temp, "\r\n");
+
+    fi = fdopen(sock, "r");
+    if (write(sock, temp, strlen(temp)) < 0)
+	err_sys("write");
+    while (fgets(buf, sizeof(buf), fi)) {
+	/* If there are multiple matches only the server of the first record
+	   is queried */
+	if (state == 0 &&
+    strncmp(buf, "Registrant Name:CONTACT NOT AUTHORITATIVE", 15) == 0)
+	    state = 1;
+	if (state == 1 &&
+    strncmp(buf, "Registrant Street1:Whois Server:", 16) == 0) {
+	    char *p, *q;
+
+	    for (p = buf; *p != ':'; p++);	/* skip until colon */
+	    for (p++; *p != ':'; p++);		/* skip until 2nd colon */
+	    for (p++; *p == ' '; p++);		/* skip colon and spaces */
+	    ret = malloc(strlen(p) + 1);
 	    for (q = ret; *p != '\n' && *p != '\r'; *q++ = *p++); /*copy data*/
 	    *q = '\0';
 	    state = 2;
 	}
-	fputs(buf, stdout);
+	if (verb)
+	    fputs(buf, stdout);
     }
     if (ferror(fi))
 	err_sys("fgets");
